diff -u4 -r dovecot-lda.ORIG/src/common.h dovecot-lda/src/common.h
--- dovecot-lda.ORIG/src/common.h	2005-08-24 17:21:13.000000000 +0200
+++ dovecot-lda/src/common.h	2005-09-01 15:50:08.000000000 +0200
@@ -8,8 +8,15 @@
 	const char *hostname;
 	const char *postmaster_address;
 };
 
+enum Operation_Mode {
+	OM_DEFAULT = 0
+	, OM_SENDMAIL
+};
+extern enum Operation_Mode operation_mode;
+extern int tempfail;
+
 extern struct sieve_settings *sieve_set;
 
 int dovecot_sieve_run(struct mail_storage *storage, struct mail *mail,
 		      const char *script_path, const char *username);
diff -u4 -r dovecot-lda.ORIG/src/deliver.c dovecot-lda/src/deliver.c
--- dovecot-lda.ORIG/src/deliver.c	2005-08-24 17:21:13.000000000 +0200
+++ dovecot-lda/src/deliver.c	2005-09-01 15:56:34.000000000 +0200
@@ -55,8 +55,19 @@
 struct et_list *_et_list = NULL;
 
 void (*hook_mail_storage_created)(struct mail_storage *storage) = NULL;
 
+/* Note: The order MUST match the enums of Operation_Mode
+	in common.h */
+static const char *operation_modes[] = {
+	"default"
+	, "sendmail"
+	, NULL
+};
+enum Operation_Mode operation_mode = OM_DEFAULT;
+
+int tempfail = 0;
+
 static struct module *modules;
 static struct ioloop *ioloop;
 static int return_value = EX_SOFTWARE;
 
@@ -396,9 +407,11 @@
 }
 
 static void print_help(void)
 {
-	printf("Usage: deliver [-c <config file>] [-d <destination user>]\n");
+	printf("Usage: deliver [-m <operation mode>] [-c <config file>] [-f] [-d <destination user>]\n");
+	puts("\toperation mode: default, sendmail");
+	puts("\t-f: Tempfail rather than return permanent error");
 }
 
 int main(int argc, char *argv[])
 {
@@ -437,8 +450,29 @@
 				i_fatal_status(EX_USAGE,
 					"Missing config file path argument");
 			}
 			config_path = argv[i];
+		} else if (strcmp(argv[i], "-f") == 0) {
+			/* tempfail rather than permanent error */
+			tempfail = 1;
+		} else if (strcmp(argv[i], "-m") == 0) {
+			/* operation mode */
+			i++;
+			if (i == argc) {
+				i_fatal_status(EX_USAGE,
+					"Missing operation mode argument");
+			}
+
+			{	int j;
+				for(j = 0; operation_modes[j]; ++j) {
+					if(strcmp(operation_modes[j], argv[i]) == 0) {
+						operation_mode = j;
+						goto om_done;
+					}
+				}
+				i_fatal_status(EX_USAGE, "Invalid operation mode: %s", argv[i]);
+				om_done: ;
+			}
 		} else {
 			print_help();
 			i_fatal_status(EX_USAGE,
 				       "Unknown argument: %s", argv[1]);
@@ -454,9 +488,9 @@
 			auth_socket = DEFAULT_AUTH_SOCKET_PATH;
 
 		ret = user_init(auth_socket, destination);
 		if (ret != 0)
-			return ret;
+			return tempfail? EX_TEMPFAIL: ret;
 
 		home = getenv("HOME");
 		if (home != NULL) {
 			/* If possible chdir to home directory so core file
@@ -570,8 +604,11 @@
 		if (save_mail(storage, "INBOX", mail) < 0) {
 			const char *error;
 			int ret, syntax, temporary_error;
 
+			if(tempfail)
+				return EX_TEMPFAIL;
+
 			error = mail_storage_get_last_error(storage, &syntax,
 							    &temporary_error);
 			if (temporary_error)
 				return EX_TEMPFAIL;
diff -u4 -r dovecot-lda.ORIG/src/mail-send.c dovecot-lda/src/mail-send.c
--- dovecot-lda.ORIG/src/mail-send.c	2005-08-24 17:21:13.000000000 +0200
+++ dovecot-lda/src/mail-send.c	2005-09-05 15:12:42.952564336 +0200
@@ -10,8 +10,9 @@
 #include "istream-header-filter.h"
 #include "smtpclient.h"
 #include "mail-send.h"
 
+#include <sysexits.h>
 #include <sys/wait.h>
 
 int global_outgoing_count = 0;
 
@@ -28,8 +29,19 @@
     int ret, sm_stat;
     pid_t sm_pid;
     size_t size;
 
+	if(operation_mode == OM_SENDMAIL) {
+		/* Let the current sendmail process handle the error */
+		fputs(reason, stderr);		/* sendmail catches this channel */
+		fputc('\n', stderr);		/* and adds it into the DSN */
+		return tempfail? EX_TEMPFAIL: EX_UNAVAILABLE;	/* FIXME/TODO: What error to return? 
+			Perhaps: EX_NOPERM ?
+
+		Better: In some cases, like out-of-quota
+			one should return a TEMPFAIL!!! ska*/
+	}
+
     origid = mail_get_first_header(mail, "Message-ID");
     rejto = get_return_path(mail);
     origreceip = mail_get_first_header(mail, "original-recipient");
 
